import { IOpt1, IOptUserBase } from './interfaces/fetchOptions';
export interface Log {
    error: (err: Error | string, name: string) => void;
}
declare const interceptors: {
    request: {
        /**
         * context 服务端如果使用了getInitialProps提供的fetch
         *
         * 那么这里的context将不为null
         */
        use: (func: (request: Request, context: any | null, log: Log) => Request) => void;
    };
    response: {
        /**
         * 回调函数支持 async await
         *
         * @data award-fetch处理后的数据结构，比如`json()`、`text()`
         *
         * @response 请求返回的响应的对象原型 Response
         *
         * @log 输出日志，log.error
         */
        use: (func: (data: any, response: Response, log: Log) => Response | Promise<Response>) => void;
    };
};
/**
 * award统一请求接口函数实现
 * ```
 * import fetch from 'award-fetch'
 *
 * fetch({
 *   url: "/api",
 *   // 劫持单个请求的的response处理
 *   transformResponse: () => {
 *
 *   }
 * })
 * ```
 *
 * ## 请求劫持处理
 *
 * ```
 * // request 表示请求参数
 * // log是函数，可以在服务器输出错误 log.error
 * fetch.interceptors.request.use((request, log) => {
 *  console.log(request);
 *  return request
 * })
 *
 * // 响应劫持处理
 * // data表示award-fetch处理的数据结果
 * // response 表示响应返回的对象原型
 * // log是函数，可以在服务器输出错误 log.error
 * fetch.interceptors.response.use((data, response, log) => {
 *  console.log("response status",response.status);
 *  return data
 * })
 * ```
 *
 * ## 取消fetch，目前仅支持xhr取消
 * ```
 *
 * const source = fetch.source();
 * // 发起上传请求
 * fetch({
 *  url: '/api/upload',
 *  method: 'POST',
 *  xhr: true,
 *  data: fileInfo,
 *  cancelToken: source.token
 * });
 *
 * // 终止上传
 * source.cancel();
 * ```
 */
declare function awardFetch(this: any, options: string | IOpt1, otherOptions?: IOptUserBase): Promise<any>;
declare namespace awardFetch {
    var interceptors: {
        request: {
            /**
             * context 服务端如果使用了getInitialProps提供的fetch
             *
             * 那么这里的context将不为null
             */
            use: (func: (request: Request, context: any, log: Log) => Request) => void;
        };
        response: {
            /**
             * 回调函数支持 async await
             *
             * @data award-fetch处理后的数据结构，比如`json()`、`text()`
             *
             * @response 请求返回的响应的对象原型 Response
             *
             * @log 输出日志，log.error
             */
            use: (func: (data: any, response: Response, log: Log) => Response | Promise<Response>) => void;
        };
    };
    var all: typeof import(".").all;
    var source: () => {
        token: {
            promise: Promise<unknown>;
        };
        cancel: (msg?: string | undefined) => void;
    };
    var setLog: (customLog: {
        error: Function;
    }) => void;
    var COMMON_ERROR: string;
    var ABORT_ERROR: string;
    var basename: boolean;
    var clean: (filepath: string) => void;
}
declare function all(fetches: any): any;
declare const source: () => {
    token: {
        promise: Promise<unknown>;
    };
    cancel: (msg?: string | undefined) => void;
};
declare const setLog: (customLog: {
    error: Function;
}) => void;
export { interceptors, all, source, setLog };
export default awardFetch;
